/*-
 * Copyright (C) @BABOLO  2004 Nov 18
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2004 Nov 18\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: cmd_file.c,v 1.18 2007/12/05 09:29:34 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sys/wait.h>
#include <sysexits.h>
#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <getCGIparm.h>
#include <mife.h>
#include <err.h>
#include <multilar.h>
#include <libpq-fe.h>
#include <babolo/BLINflag.h>
#include "pgoblin.h"
#include "pgob.h"

int
pgoblin_echo(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;

    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_echo %d %d\n", r[PGO_COUT].r, r[PGO_CCTL].r);
    if  (r[PGO_COUT].r != r[PGO_CCTL].r) {
        if  (IO_CTL->flags & PGOBLIN_FREETEXT) {
            if  (!(IO_OUT->text = strdup(IO_CTL->text))) {
                ifBLIN_QV1(options->flags) warnx("No mem echo");
                ex = EX_OSERR;
                errno = ENOMEM;
            } else 
            IO_OUT->flags |= PGOBLIN_FREETEXT;
        } else {
            IO_OUT->text = IO_CTL->text;
            IO_OUT->flags &= ~PGOBLIN_FREETEXT;
        }
        IO_OUT->length = 0;
        IO_OUT->flags |= PGOBLIN_TEXTPARM;
        IO_OUT->flags &= ~PGOBLIN_BINPARM;
    }
    IO_OUT->flags |= PGOBLIN_TEXTPARM;
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_echo %d\n", ex);
    return(ex);
}

int
pgoblin_file(pgoblin_main *options, pgoblin_r *r) {
    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_file %d %d\n", r[PGO_COUT].r, r[PGO_CCTL].r);
    if  (IO_CTL->flags & PGOBLIN_FREETEXT) {
        IO_OUT->path = strdup(IO_CTL->text);
        IO_OUT->flags |= PGOBLIN_FREEPATH;
    } else {
        IO_OUT->path = IO_CTL->text;
        IO_OUT->flags &= ~PGOBLIN_FREEPATH;
    }
    IO_OUT->flags |= PGOBLIN_FILEOVER | PGOBLIN_PATHPARM;
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_file\n");
    return(EX_OK);
}

int
pgoblin_cat(pgoblin_main *options, pgoblin_r *r) {
    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_cat %d %d\n", r[PGO_COUT].r, r[PGO_CCTL].r);
    if  (IO_CTL->flags & PGOBLIN_FREETEXT) {
        IO_OUT->path = strdup(IO_CTL->text);
        IO_OUT->flags |= PGOBLIN_FREEPATH;
    } else {
        IO_OUT->path = IO_CTL->text;
        IO_OUT->flags &= ~PGOBLIN_FREEPATH;
    }
    IO_OUT->flags |= PGOBLIN_PATHPARM;
    IO_OUT->flags &= ~PGOBLIN_FILEOVER;
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_cat\n");
    return(EX_OK);
}

int
pgoblin_getarg(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;

    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_getarg %d\n", r[PGO_COUT].r);
    if  ((ex = getCGIparmarray( 0, (u_char **)options->argv, "any", "\t", "\\N"
                              , IO_OUT->wri
                              , IO_OUT->onu
        ))                    ) {
        ifBLIN_QV1(options->flags) warn("getCGIparmarray");
    }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getarg %d\n", ex);
    return(ex);
}

int
pgoblin_getenv(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;

    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_getenv %d\n", r[PGO_COUT].r);
    if  ((ex = getCGIparmenv( gCGI_RNUM, (const u_char **)options->envp, "any", "\t", "\\N"
                            , IO_OUT->wri
                            , IO_OUT->onu
        ))                  ) {
        ifBLIN_QV1(options->flags) warn("getCGIparmenv");
    }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getenv %d\n", ex);
    return(ex);
}

int
pgoblin_getget(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;

    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_getget %d\n", r[PGO_COUT].r);
    if  ((ex = getCGIparmcopy( gCGI_RNUM|gCGI_SEPR, getenv("QUERY_STRING"), "any", "\t", "\\N"
                             , IO_OUT->wri
                             , IO_OUT->onu
        ))                   ) {
        ifBLIN_QV1(options->flags) warn("getget getCGIparmcopy");
    }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getget %d\n", ex);
    return(ex);
}

int
pgoblin_getpost(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;
    mife_descriptor *in;

    in = IO_IN->mife;
    ifBLIN_QV4(options->flags)
        fprintf(stderr, "+pgoblin_getpost %d in %s\n", r[PGO_COUT].r, in ? "OK" : "NULL");
    if  (mife_read(in, 0, 0) < 0) {
        ifBLIN_QV1(options->flags) warn("getpost 1");
        return(EX_OSERR);
    }
    if  ((ex = getCGIparmcopy( gCGI_RNUM|gCGI_SEPR, mife_get(in, 0), "any", "\t", "\\N"
                             , IO_OUT->wri
                             , IO_OUT->onu
        ))                   ) {
        ifBLIN_QV1(options->flags) warn("getpost getCGIparmcopy");
    }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getpost %d\n", ex);
    return(ex);
}

int
pgoblin_getproc(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;

    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_getproc %d\n", r[PGO_COUT].r);
    if  ((ex = getCGIparmproc( 0, "any", "\t", "\\N"
                             , IO_OUT->wri
                             , IO_OUT->onu
        ))                   ) {
        ifBLIN_QV1(options->flags) warn("getCGIparmproc");
    }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getproc %d\n", ex);
    return(ex);
}

int
pgoblin_getjob(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK;
    pgoblin_job *j;
    size_t i;
    FILE *fi;

    ifBLIN_QV4(options->flags) fprintf(stderr, "+pgoblin_getjob %d\n", r[PGO_CJOB].r);
    if  (r[PGO_CJOB].g.j) {
        if  (!(fi = funopen( (void*)(IO_OUT->onu)
                           , NULL
                           , (int (*)(void*, const char*, int))mife_writ
                           , NULL
                           , NULL
            ) )            ) {
            ifBLIN_QV1(options->flags) warn("funopen on %d", r[PGO_CJOB].r);
            ex = EX_OSERR;
        } else {
            for (i = 0; i <  MULAR_NEXT(*r[PGO_CJOB].g.j); i++) {
                j = mular_getix(*r[PGO_CJOB].g.j, i);
                if  (!(j->pid)) {
                    ifBLIN_QV1(options->flags) warnx("No pid in %d", j->cmdn);
                    errno = EINVAL;
                    ex = EX_SOFTWARE;
                } else if ((j->cmdn >= 0) && (j->status == PGOBLIN_NOSTATUS)) {
                    if  (((ex = waitpid(j->pid, &(j->status), WNOHANG)) == j->pid) || !ex) {
                        ex = EX_OK;
                    } else if (ex > 0) {
                        ifBLIN_QV1(options->flags) warn("Pid %d<>%d in %d", ex, j->pid, j->cmdn);
                        ex = EX_OSERR;
                    } else if (errno == ECHILD || errno == EDEADLK || errno == EAGAIN) {
                        ex = EX_OK;
                    } else {
                        ifBLIN_QV1(options->flags) warn("Pid %d in %d", j->pid, j->cmdn);
                        ex = EX_OSERR;
                }   }
                fprintf(fi, "%d	%d	%d	%d\n", i, j->cmdn, j->pid, j->status);
            }
            fprintf(fi, "\\.\n");
            if  (fclose(fi) < 0) {
                ifBLIN_QV1(options->flags) warn("fclose on %d", r[PGO_CJOB].r);
                ex = EX_OSERR;
    }   }   }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getjob %d\n", ex);
    return(ex);
}

int
pgoblin_getspq(pgoblin_main *options, pgoblin_r *r) {
    return(EX_UNAVAILABLE);
}

int
pgoblin_getstr(pgoblin_main *options, pgoblin_r *r) {
    int ex = EX_OK, l, j;
    FILE *fi;
    mife_descriptor *in;
    off_t off;
    char *cont;
    ssize_t bl;
    size_t i;

    in = IO_IN->mife;
    ifBLIN_QV4(options->flags)
        fprintf(stderr, "+pgoblin_getstr %d in %s\n", r[PGO_COUT].r, in ? "OK" : "NULL");
    if  (!(fi = funopen( (void*)(IO_OUT->onu)
                       , NULL
                       , (int (*)(void*, const char*, int))mife_writ
                       , NULL
                       , NULL
        ) )            ) {
        ifBLIN_QV1(options->flags) warn("funopen on %d", r[PGO_COUT].r);
        ex = EX_OSERR;
    } else {
        for (off = 0, j = 0, l = 1; !(in->flag & MIFE_EOFL); off += bl) {
            if  ((bl = mife_read(in, 1 << (PGOBLIN_RAWIOBUFL + 16), off)) < 0) {
                ifBLIN_QV1(options->flags) warn("getstr mife_read");
                ex = EX_IOERR;
                break;
            }
            if  (!(cont = mife_get(in, off))) {
                ifBLIN_QV1(options->flags) warn("getstr mife_get");
                ex = EX_IOERR;
                break;
            }
            for (i = 0; i < bl; i++, cont++) {
                switch (*cont) {
                    case '\\': if (l) fprintf(fi, "%d\t", j); l = 0; fprintf(fi, "\\\\");      break;
                    case '\0': if (l) fprintf(fi, "%d\t", j); l = 0; fprintf(fi, "\\0");       break;
                    case '\t': if (l) fprintf(fi, "%d\t", j); l = 0; fprintf(fi, "\\t");       break;
                    case '\n': if (l) fprintf(fi, "%d\t", j); l = 1; fprintf(fi, "\n"); j++;   break;
                    default:   if (l) fprintf(fi, "%d\t", j); l = 0; fprintf(fi, "%c", *cont); break;
        }   }   }
        if  (!l) fprintf(fi, "\n");
        fprintf(fi, "\\.\n");
    }
    if  (fclose(fi) < 0) {
        ifBLIN_QV1(options->flags) warn("fclose on %d", r[PGO_COUT].r);
        ex = EX_OSERR;
    }
    ifBLIN_QV4(options->flags) fprintf(stderr, "-pgoblin_getstr %d\n", ex);
    return(ex);
}
